require 'socket'      # Sockets are in standard library
require 'faraday'
require 'json'

module Stormpack
  module Pusher
    class Client
      attr_accessor :host, :port

      def initialize(host: 'localhost', port: 1234)
        self.host = host
        self.port = port
      end

      def register_http(url: self.host, auth_token: 'ultravisor.core', filters: [], secret_token: '1234', destination: 'http://google.com')
        conn = Faraday.new(:url => url)
        conn.post("subscribe/callback?token=#{auth_token}") do |req|
          req.body = { filters: filters, token: secret_token, callback: destination}.to_json
        end
      end

      def publish(data, tags = ['all'], wait=false)
        submit "publish #{tags.join(',')} #{data}", wait
      end

      def register(token, tags = [], wait=false)
        submit "register #{tags.join(',')} #{token}", wait
      end

      def unregister(token, wait=false)
        submit "unregister #{token}", wait
      end

      def set(token, tags = [], wait = false)
        submit "set #{tags.join(',')} #{token}", wait
      end

      def unset(token, tags = [], wait = false)
        submit "unset #{tags.join(',')} #{token}", wait
      end

      private

      def submit(command, wait)
        command = "ensure #{command}" if wait

        with_client do |cli|
          cli.puts("#{command}\n")

          if wait
            cli.gets
          else
            ''
          end

        end
      end

      def with_client
        Thread.current[:sp_client] ||= TCPSocket.new(host, port)
        if Thread.current[:sp_client].closed?
          Thread.current[:sp_client] = TCPSocket.new(host, port)
        end
        res = yield Thread.current[:sp_client]
        res
      rescue
        Thread.current[:sp_client] = TCPSocket.new(host, port)
        res = yield Thread.current[:sp_client]
        res
      end

    end
  end
end